@@ -1,100 +1,76 @@
+-----------------------------------------------------------------------------
+-- Defines the main class for Sputnik - an extensible wiki implemented in Lua.
+-- (For usage, see sputnik.wsapi_app.)
+--
+-- (c) 2007, 2008  Yuri Takhteyev (yuri@freewisdom.org)
+-- License: MIT/X, see http://sputnik.freewisdom.org/en/License
+-----------------------------------------------------------------------------
+
 module(..., package.seeall)
 
 require("md5")
+require("wsapi.util")
 require("cosmo")
+
 require("saci")
 require("sputnik.actions.wiki")
 require("sputnik.i18n")
 require("sputnik.util")
+require("sputnik.wsapi_app")
+
+local zlib_loaded, zlib = pcall(require, "zlib")
+
+new_wsapi_run_fn = sputnik.wsapi_app.new  -- for backwards compatibility
 
 -----------------------------------------------------------------------------
--- Applies default config values
+-- Applies default config values.
 -----------------------------------------------------------------------------
 local function apply_defaults(config)
-   config                  = config or {}
+   config = config or {}
    assert(config.TOKEN_SALT, "TOKEN_SALT must be set")
-   config.ROOT_PROTOTYPE   = config.ROOT_PROTOTYPE or "@Root"
-   config.PASSWORD_SALT    = config.PASSWORD_SALT or "2348979898237082394172309847123"
-   config.CONFIG_PAGE_NAME = config.CONFIG_PAGE_NAME or "sputnik/config"
-   config.PASS_PAGE_NAME   = config.PASS_PAGE_NAME or "sputnik/passwords"
-   --config.LOGGER = config.LOGGER or "file"
-   --config.LOGGER_PARAMS = config.LOGGER_PARAMS
-   --                       or {"/tmp/sputnik-log.log", "%Y-%m-%d"}
+   assert(config.BASE_URL, "BASE_URL must be set")
+
+   -- Set some defaults on the configuration table
+   local defaults = {}
+   setmetatable(config, {__index = defaults})
+
+   defaults.ROOT_PROTOTYPE          = "@Root"
+   defaults.PASSWORD_SALT           = "2348979898237082394172309847123"
+   defaults.CONFIG_PAGE_NAME        = "sputnik/config"
+
    return config
 end
 
------------------------------------------------------------------------------
--- Code to track globals
------------------------------------------------------------------------------
-if STRICT then
-   local exported={export = true}
-   local mt = getmetatable(_M) or {}
-   setmetatable(_M, mt)
-   mt.__newindex = function (t, n, v)
-     if not exported[n] then
-        error("assign to undeclared variable '"..n.."'", 2)
-     end
-     rawset(t, n, v)
-   end
-   function export(...)
-      for _, v in ipairs{...} do exported[v] = true end
-   end
-   export("new", "new_wsapi_run_fn")
-end
-
------------------------------------------------------------------------------
--- Sets up Sputnik in the current or specified directory
------------------------------------------------------------------------------
-function setup(dir)
-   require("sputnik.installer")
-   sputnik.installer.reset_salts()
-   sputnik.installer.make_wsapi_script(dir, "kepler/htdocs/sputnik.ws")
-   sputnik.installer.make_cgi_file(dir, "sputnik.cgi")
-end
-
 -----------------------------------------------------------------------------
 -- THE SPUTNIK CLASS  -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
 -----------------------------------------------------------------------------
 local Sputnik = {}
 local Sputnik_mt = {__metatable = {}, __index = Sputnik}
 
-
-
 -----------------------------------------------------------------------------
 -- Creates a new instance of Sputnik.
+--
+-- @param config         a table of bootstrap configurations.
+-- @param logger         a lualogging-compatible logger or nil.
+-- @return               an instance of Sputnik.
 -----------------------------------------------------------------------------
-function new(config)
+function new(config, logger)
    -- Set up default configuration variables
    config = apply_defaults(config)
    -- Create and return the new initialized Sputnik instance
-   local obj = setmetatable({}, Sputnik_mt)
+   local obj = setmetatable({logger=logger or util.make_logger()}, Sputnik_mt)
    obj:init(config)
    return obj
 end
 
-function new_wsapi_run_fn(config)
-   local my_sputnik = new(config)
-   return  function (...)
-     return my_sputnik:wsapi_run(...)
-   end
-end
-
 -----------------------------------------------------------------------------
 -- Initializes a the new Sputnik instance.
+--
+-- @param initial_config a table representing bootrastrap configurations.
 -----------------------------------------------------------------------------
 function Sputnik:init(initial_config)
    -- setup the logger -- do this before loading user configuration
    self.initial_config = initial_config -- for reloading
-   if initial_config.LOGGER then
-      require("logging."..initial_config.LOGGER)
-      self.logger = logging[initial_config.LOGGER](unpack(initial_config.LOGGER_PARAMS))
-   else
-      self.logger = {
-         debug = function(self, level, message) end, -- do nothing
-         info  = function(self, level, message) end,
-         error = function(self, level, message) end,
-      }
-   end
 
    --- Turns a string into something that can be used as a node name.
    local dirify = initial_config.DIRIFY_FN or sputnik.util.dirify
@@ -121,7 +97,7 @@ function Sputnik:init(initial_config)
          local node = self.saci:make_node(data, id)
          assert(node)
          if page_module.CREATE_DEFAULT then
-            node:save()
+            node = self:save_node(node, nil)
             node = self.saci:get_node(id)
          end
          return node
@@ -146,15 +122,11 @@ function Sputnik:init(initial_config)
    assert(config_node, "Failed to retrieve the config node "
                        ..tostring(self.config.CONFIG_PAGE_NAME))
    assert(type(config_node)=="table")
+   --print(config_node.content, "XX", config_node.fields.content.activate)
    for k,v in pairs(config_node.content) do
       self.config[k] = v
    end
 
-   -- setup markup
-
-   self.markup_module = require(self.config.MARKUP_MODULE or "sputnik.markup.markdown")
-   self.markup = self.markup_module.new(self)
-
    -- setup cache
    if self.config.CACHE_MODULE then
       local cache_mod = require(self.config.CACHE_MODULE)
@@ -183,7 +155,23 @@ function Sputnik:init(initial_config)
    -- setup authentication
    self.auth_mod = require(self.config.AUTH_MODULE or "sputnik.auth.simple")
    self.auth = self.auth_mod.new(self, self.config.AUTH_MODULE_PARAMS)
-   
+   self.saci.permission_groups.Admin = function(user)
+      return user and self.auth:get_metadata(user, "is_admin") == "true"
+   end
+
+   local groups_mt = {
+      __index = function(table, key)
+         return function (user)
+                   return user and self.auth:get_metadata(user, "is_"..key) == "true"
+                end
+      end
+   }
+   self.saci.permission_groups.is = setmetatable({}, groups_mt)
+   self.saci.permission_groups.edit_and_save = {"save", "edit", "preview"}
+   self.saci.permission_groups.show = {"show", "show_content", "cancel"}
+   self.saci.permission_groups.history_and_diff = {"history", "diff"}
+   self.saci.permission_groups.show_etc = {"show", "show_content", "cancel", "history", "diff"}
+
    -- setup wrappers
    self.wrappers = sputnik.actions.wiki.wrappers -- same for "wiki" wrappers      
 end
@@ -202,12 +190,44 @@ end
 -- @return               formatted time (string)
 -----------------------------------------------------------------------------
 function Sputnik:format_time(timestamp, format, tzoffset, tzname)
+   if type(timestamp) == "number" or timestamp:match("^[0-9]*$") then
+      timestamp = os.date("!%Y-%m-%d %H:%M:%S", timestamp)
+   end
    return versium.util.format_time(timestamp,format, 
                                    tzoffset or self.config.TIME_ZONE,
                                    tzname or self.config.TIME_ZONE_NAME)
 end
 
---- Returns a small icon for this user.
+-----------------------------------------------------------------------------
+-- Converts a versium time stamp into the format specified in RFC-822.  This
+-- format will always be given in GMT for simplicity, which will be correctly
+-- altered by the reader.
+--
+-- @param timestamp      Versium timestamp (string) 
+-- @return               Time string that complied with RFC-822 (string)
+-----------------------------------------------------------------------------
+function Sputnik:format_time_RFC822(timestamp)
+   return self:format_time(timestamp, "!%a, %d %b %Y %H:%M:%S +0000")
+end
+
+function Sputnik:get_gravatar_for_email(email)
+   if self.config.USE_GRAVATAR then
+      return "http://www.gravatar.com/avatar/"..md5.sumhexa(email)
+                .."?s=22&d="
+                ..sputnik.util.escape_url("http://"
+                    ..self.config.DOMAIN..self:make_url("icons/user", "png"))
+   else
+      return self:make_url("icons/user", "png")
+   end
+end
+
+
+-----------------------------------------------------------------------------
+-- Returns a small icon to represent a given user.
+--
+-- @param user           username
+-- @return               a url of an icon
+-----------------------------------------------------------------------------
 function Sputnik:get_user_icon(user)
    self.user_icon_hash = self.user_icon_hash or {}
    if self.user_icon_hash[user] then
@@ -222,62 +242,124 @@ function Sputnik:get_user_icon(user)
       icon = self:make_url("icons/system", "png") 
    elseif self.auth:user_exists(user) then
       local email = self.auth:get_metadata(user, "email")
-      if email and self.config.USE_GRAVATAR then 
-         icon = "http://www.gravatar.com/avatar/"..md5.sumhexa(email)
-                .."?s=16&amp;d=http://"
-                ..self.config.DOMAIN..self:make_url("icons/user", "png")
+      if email then 
+         icon = self:get_gravatar_for_email(email)
       end
    end
+   if (not icon) and user:match("@") then
+      icon = self:get_gravatar_for_email(user)
+   end
    self.user_icon_hash[user] = icon or self:make_url("icons/user", "png")
    return self:escape(self.user_icon_hash[user])
 end
 
---- Escapes a text for using in a textarea.
-function Sputnik.escape(self, text)
+-----------------------------------------------------------------------------
+-- Escapes a text for using in a textarea.
+--
+-- @param text           text to be escaped.
+-- @return               escaped text.
+-----------------------------------------------------------------------------
+function Sputnik:escape(text)
    return sputnik.util.escape(text)
 end
 
---- Escapes a URL.
-function Sputnik.escape_url (self, text)
-   return sputnik.util.escape_url(text)
-end
-
-function Sputnik:node_exists(id)
-   id = self:dirify(id)
-   return self.saci:node_exists(id) or pcall(require, "sputnik.node_defaults."..id)
+-----------------------------------------------------------------------------
+-- Escapes a URL.
+--
+-- @param url            a URL to be escaped.
+-- @return               the escaped URL
+-----------------------------------------------------------------------------
+function Sputnik:escape_url (url)
+   return sputnik.util.escape_url(url)
 end
 
 -----------------------------------------------------------------------------
---- Makes a URL from a table of parameters.
+-- Checks if a node with the given ID exists.
+--
+-- @param id             the id of a node.
+-- @return               true or false.
+-----------------------------------------------------------------------------
+function Sputnik:node_exists(id)
+   id = self:dirify(id)
+   return self.saci:node_exists(id) or pcall(require, 
+                                             "sputnik.node_defaults."..id)
+end
+
+-----------------------------------------------------------------------------
+-- Makes a URL from a table of parameters.
+--
+-- @param node_name      a node id.
+-- @param action         action/command as a string.
+-- @param params         query parameters
+-- @param anchor         link anchor
+-- @return               a URL.
 -----------------------------------------------------------------------------
 function Sputnik:make_url(node_name, action, params, anchor)
    if not node_name or node_name=="" then
       node_name = self.config.HOME_PAGE
    end
-   node_name = self:dirify(node_name)
-   if action and action~="show" then 
-      node_name = node_name.."."..action
+
+   local interwiki_code = node_name:match("^([^%:]*)%:")
+   local interwiki_handler = self.config.INTERWIKI[interwiki_code]
+   if interwiki_handler then
+      node_name = node_name:gsub("^[^%:]*%:", "") -- crop the code
    end
-   if anchor then
-      anchor = "#"..anchor
-   else
-      anchor = ""
-   end
-   if params and next(params) then
-      local link = self.config.BASE_URL.."?p="..node_name
-      for k, v in pairs(params or {}) do
-         link = link.."&"..k.."="..(v or "")
+
+   local dirified = self:dirify(node_name) --wsapi.util.url_encode()
+
+   local url
+
+   -- first the node name
+   if interwiki_handler then
+      local handler_type = type(interwiki_handler)
+      if handler_type == "string" then
+         if interwiki_handler:match("%%s") then
+            url = string.format(interwiki_handler, node_name)
+         else
+            url = interwiki_handler..node_name
+         end
+      elseif handler_type == "function" then
+         url = interwiki_handler(node_name)
+      else
+         error("Interwiki handler should be string or function, but is "..handler_type)
       end
-      return self:escape(link..anchor)
-   elseif node_name==self.config.HOME_PAGE then
-      return self:escape(self.config.HOME_PAGE_URL..anchor)
    else
-      return self:escape(self.config.NICE_URL..node_name..anchor)
-   end   
+      url = self.config.NICE_URL..dirified
+   end
+
+   -- then the action and HOME_PAGE
+   if action and action~="show" then 
+      url = url.."."..action
+   elseif dirified==self.config.HOME_PAGE and #(params or {})==0 then
+      url = self.config.HOME_PAGE_URL
+   end
+
+   -- then the parameters
+   if params and next(params) then
+      for k, v in pairs(params or {}) do
+         if k~="p" then
+            url = url.."&"..wsapi.util.url_encode(k).."="
+                          ..wsapi.util.url_encode(v or "")
+         end
+      end
+   end
+
+   -- finally the anchor
+   if anchor then
+      url = url.."#"..anchor
+   end
+
+   return self:escape(url), node_name
 end
 
 -----------------------------------------------------------------------------
---- Makes a link from a table of parameters.
+-- Makes a link from a table of parameters.
+--
+-- @param node_name      a node id.
+-- @param action         action/command as a string.
+-- @param params         query parameters
+-- @param anchor         link anchor
+-- @return               a URL.
 -----------------------------------------------------------------------------
 function Sputnik:make_link(node_name, action, params, anchor, options)
    assert(node_name)
@@ -286,7 +368,7 @@ function Sputnik:make_link(node_name, action, params, anchor, options)
    if node_name:find("%.") then 
       node_name, action = node_name:match("(.+)%.(.+)")
    end
-   local css_class = "local"
+   --local css_class = "local"
    local url = self:make_url(node_name, action, params, anchor)
    self.logger:debug("Creating a link to "..node_name)
    if (not options.mark_missing==false)
@@ -295,32 +377,42 @@ function Sputnik:make_link(node_name, action, params, anchor, options)
       url = self:make_url(node_name, action, params, anchor)
       self.logger:debug("No such node, will link to .edit")
    end
-   return string.format("href='%s' class='%s'", url, css_class)
+   --return string.format("href='%s' class='%s'", url, css_class)
+   return string.format("href='%s'", url, css_class)
 end
 
 -----------------------------------------------------------------------------
---- Does a bit of extra activation beyond what SACI does.
+-- Further "activates" a node for use in Sputnik.
+--
+-- @param node           a Saci node.
+-- @return               the same node with some extra methods added.
 -----------------------------------------------------------------------------
-function Sputnik:activate_node(node, params)
+function Sputnik:activate_node(node)
 
    -- setup the page-specific translator
    for i, translation_node in ipairs(node.translations) do
       local translations = self:get_node(translation_node).content
-      assert(type(translations) == "table", "Could not load translation node")
+      assert(type(translations) == "table",
+             "Could not load translation node")
       for k, translation in pairs(translations) do
          node.translations[k] = translation
       end
     end
     node.translator = sputnik.i18n.make_translator(node.translations,
-                                                   self.config.INTERFACE_LANGUAGE)
+                                              self.config.INTERFACE_LANGUAGE)
     
    -- translate the templates
-   for i, template_node in ipairs(node.templates) do
-      local templates = self:get_node(template_node).content
+   for i, template_node_id in ipairs(node.templates) do
+      local template_node = self:get_node(template_node_id)
+      local templates = template_node.content
+      --templates.MAIN = template_node.main
+      --templates.HEAD = template_node.head
+      --templates.BODY = template_node.body
       assert(type(templates) == "table", "Could not load templates node")
       for k, template in pairs(templates) do
          node.templates[k] = node.translator.translate(template)
       end
+      
    end
    
    -- load the actions (turn them into callable functions)
@@ -337,78 +429,13 @@ function Sputnik:activate_node(node, params)
    end
    local action_loader = action_loader()
    
-   for k, v in pairs(node.actions) do
-      local mod_name, dot_action = sputnik.util.split(v, "%.")
-      node.actions[k] = action_loader.load(mod_name)[dot_action]
+   for command, action_function in pairs(node.actions) do
+      if type(action_function) == "string" then
+         local mod_name, dot_action = sputnik.util.split(action_function, "%.")
+         node.actions[command] = action_loader.load(mod_name)[dot_action]
+      end
    end
-   
-   -- create a function to check permissions ---------------------------
-   node.check_permissions = function(node, user, action)
-      local all_users = {}
-      local all_actions = {}
-      local has_permission = true
 
-      -- This function handles toggling the actual allow state
-      local function set(suser, saction, svalue)
-         local is_user, is_action
-
-         -- Resolve the user side of the permission
-         if type(suser) == "function" then
-            is_user = suser(user, self.auth)
-         else
-            is_user = (suser == user) or (suser == all_users)
-         end
-
-         -- Resolve the action
-         if type(saction) == "function" then
-            is_action = saction(action)
-         else
-            is_action = (saction == action) or (saction == all_actions)
-         end
-
-         if is_user and is_action then
-            has_permission = svalue
-         end
-      end
-
-      local function allow(suser, saction) 
-         set(suser, saction, true)
-      end
-      local function deny(suser, saction)
-         set(suser, saction, false)
-      end
-      if node.permissions then
-         local sandbox = {
-            all_users = all_users,
-            all_actions = all_actions,
-            allow = function(suser, saction)
-               set(suser, saction, true)
-            end,
-            deny = function(suser, saction)
-               set(suser, saction, false)
-            end,
-            Authenticated = function(user, auth)
-               return user ~= nil
-            end,
-            Anonymous = function(user, auth)
-               return not user
-            end,
-            Admin = function(user, auth)
-               if user then
-                  return auth:get_metadata(user, "is_admin") == "true"
-               else
-                  return false
-               end
-            end,
-         }
-         local func = assert(loadstring(node.permissions))
-         setfenv(func, sandbox)
-         local succ,err = assert(pcall(func))
-      end
-
-      return has_permission
-   end     
-   
    -- set wrappers -----------------------------------------------------
    node.wrappers = self.wrappers
    
@@ -427,18 +454,25 @@ function Sputnik:get_node(id, version, mode)
       node.title = temp_title
       node.raw_values.title = temp_title
    end
-   if mode~="basic" then
-      self:prime_node(node)
-   end
    return node, stub
 end
 
 -----------------------------------------------------------------------------
 -- Adds extra sputnik-specific fields to a node.
 -----------------------------------------------------------------------------
-function Sputnik:prime_node(node)
-   node.markup = self.markup
-   self:add_links_and_urls(node)
+function Sputnik:decorate_node(node)
+   -- Determine which markup module the node should be using based on
+   -- the markup_module field, or the default
+   local markup_module_name
+   if node.markup_module and node.markup_module:len() > 0 then
+      markup_module_name = "sputnik.markup."..node.markup_module
+   else
+      markup_module_name = "sputnik.markup.markdown"
+   end
+
+   local markup_module = require(markup_module_name)
+   node.markup = markup_module.new(self)
+
    node.messages = {}
    for i, class in ipairs{"error", "warning", "success", "notice"} do
       node["post_"..class] = function(self, message) 
@@ -448,7 +482,9 @@ function Sputnik:prime_node(node)
    -- Table/Function that allow the developer to add custom HTML response headers
    node.headers = {}
    node.add_header = function(self, header, value) self.headers[header] = value end
-   node.redirect = function(self, url) self.headers["Location"] = url end
+   node.redirect = function(node_self, url)
+      node_self.headers["Location"] = url
+   end
 
    node.css_links = {}
    node.css_snippets = {}
@@ -469,7 +505,7 @@ function Sputnik:prime_node(node)
       return add(self.css_snippets, href.."|"..media, {snippet = snippet, media = media})
    end
    function node:add_javascript_link(href)
-      return add(self.javascript_links, href, {href=href})
+      add(self.javascript_links, href, {href=href})
    end
    function node:add_javascript_snippet(snippet)
       return add(self.javascript_snippets, snippet, {snippet=snippet})
@@ -477,51 +513,45 @@ function Sputnik:prime_node(node)
    return node
 end  
 
------------------------------------------------------------------------------
--- Makes node.urls:foo(params) equivalent to sputnik:make_url(node.name, "foo", 
--- params) and node.links:foo(params) equivalent to sputnik:make_link(
--- node.name, "foo", params) for ANY foo.
------------------------------------------------------------------------------
-function Sputnik:add_links_and_urls(node)
-
-   local make_fn = function (method, key)
-      return function(inner_self, params)
-                -- method is either self.make_link or self.make_url
-                return method(self, node.name, key, params, nil, 
-                              {mark_missing=false})
-      end
-   end
-   node.urls  = { __index = function(table, key) return make_fn(self.make_url, key) end }
-   setmetatable(node.urls, node.urls)
-   node.links = { __index = function(table, key) return make_fn(self.make_link, key) end }
-   setmetatable(node.links, node.links)
-   return node
-end
-
------------------------------------------------------------------------------
--- Generates a node-like table to make urls.
------------------------------------------------------------------------------
-function Sputnik.pseudo_node(self, node_name)
-   local node = {name = node_name }
-   self:add_links_and_urls(node)
-   return node
-end
-
 -----------------------------------------------------------------------------
 --- Updates node with values from params table.
 -----------------------------------------------------------------------------
 function Sputnik:update_node_with_params(node, params)
    node:update(params, node.fields)
    --new_node.name = node.name
-   self:prime_node(node)
+   self:decorate_node(node)
    return node
 end
 
+-----------------------------------------------------------------------------
+-- This function is provided as a wrapper to node:save(...) that respects the
+-- save_hook field in a node.  This field defines a module and function name
+-- that are called when the node is being saved, allowing for last minute
+-- changes to the node parameters, or for other actions to be taken alongside
+-- the save.
+--
+-- NODE: A save hook is passed the node, the request object and the sputnik
+-- instance, although request and sputnik are not guaranteed to be available
+-- for certain types of nodes.
+-----------------------------------------------------------------------------
+function Sputnik:save_node(node, request, ...)
+   local new_node = node
+   if type(node.save_hook) == "string" and #node.save_hook > 0 then
+      local mod_name, func_name = sputnik.util.split(node.save_hook, "%.")
+      local module = require("sputnik.hooks." .. mod_name)
+      local save_hook = module[func_name]
+      new_node = save_hook(node, request, self)
+   end
+   -- Actually perform the save at the saci layer
+   new_node:save(...)
+   return new_node
+end
+
 -----------------------------------------------------------------------------
 -- Returns node's history.
 -----------------------------------------------------------------------------
-function Sputnik:get_history(node_name, limit, date)
-   local edits = self.saci:get_node_history(node_name, date, limit)
+function Sputnik:get_history(node_name, limit, date_prefix)
+   local edits = self.saci:get_node_history(node_name, date_prefix, limit)
    if limit then 
       for i=limit, #edits do
          table.remove(edits, i)
@@ -533,26 +563,8 @@ end
 -----------------------------------------------------------------------------
 -- Returns history for all nodes.
 -----------------------------------------------------------------------------
-function Sputnik:get_complete_history(limit, date)
-   local edits = {}
-   for i, id in ipairs(self:get_node_names()) do
-      local node = Sputnik.pseudo_node(self, id)
-      node.id = id
-      for i, edit in ipairs(self:get_history(id, limit, date)) do
-         edit.id = id
-         edit.node = node
-         table.insert(edits, edit)
-      end
-   end
-   table.sort(edits, function(e1, e2) return e1.timestamp > e2.timestamp end)
-   if limit then
-      local another_table = {}
-      for i=1,limit do
-         table.insert(another_table, edits[i])
-      end
-      edits = another_table
-   end   
-   return edits
+function Sputnik:get_complete_history(limit, date_prefix, id_prefix)
+   return self.saci:get_complete_history(id_prefix, date_prefix, limit)
 end
 
 -----------------------------------------------------------------------------
@@ -594,7 +606,7 @@ function Sputnik:get_uid(namespace, type)
    local node_name = "_uid:" .. namespace
    local node = self:get_node(node_name)
    node = self:update_node_with_params(node, {content=hash})
-   node:save("Sputnik-UID", hash)
+   node = self:save_node(node, nil, "Sputnik-UID", hash)
 
    -- Retrieve the node history 
    local history = self:get_history(node_name)
@@ -664,7 +676,7 @@ function Sputnik:translate_request (request)
       new_params.post_timestamp = request.params.post_timestamp
       request.params = new_params
    end
-   
+
    -- break "p" parameter into node name and the action
    if request.params.p and request.params.p~="" then
       request.node_name, request.action = sputnik.util.split(request.params.p, "%.")
@@ -683,7 +695,13 @@ function Sputnik:translate_request (request)
                                                       request.params.password)
       if not request.user then
          request.auth_message = "INCORRECT_PASSWORD"
+         -- TODO: I am unsure what the behavior here should be.. 
+         request.node_name = self.config.LOGIN_NODE
       else
+         if request.params.next then
+            request.redirect = self:make_url(request.params.next)
+         end
+
          self.logger:debug(request.user..","..request.auth_token)
       end
    else
@@ -701,15 +719,42 @@ end
 
 -----------------------------------------------------------------------------
 -- Handles a request.
+--
+-- @param request        a WSAPI request table.
+-- @param response       a WSAPI response table.
+-- @return               the response
 -----------------------------------------------------------------------------
-function Sputnik:run(request, response)
+function Sputnik:handle_request(request, response)
    self.auth = self.auth_mod.new(self, self.config.AUTH_MODULE_PARAMS)
-   if self.saci.reset_cache then self.saci:reset_cache() end
 
-   self.cookie_name = "Sputnik_"..md5.sumhexa(self.config.BASE_URL)
+   self.cookie_name = self.config.COOKIE_NAME.."_"..md5.sumhexa(self.config.BASE_URL)
    request = self:translate_request(request)
+   request.is_indexable = true
+
+   local dirified = self:dirify(request.node_name)
 
    local node, stub = self:get_node(request.node_name, request.params.version)
+   node.is_a_stub = stub
+   self:decorate_node(node)
+
+   if request.redirect then
+      self.logger:debug("Sending redirect")
+      -- Set the authentication cookie so it's not lost
+      local cookie_value = (request.user or "").."|"..(request.auth_token or "")
+      response:set_cookie(self.cookie_name, {value=cookie_value, path="/"})
+
+      response.headers["Content-Type"] = "text/html"
+      response.headers["Location"] = request.redirect
+      response:write("redirect")
+      return response
+   end
+
+   if dirified ~= request.node_name then
+      response.headers["Content-Type"] = content_type or "text/html"
+      response.headers["Location"] = self:make_url(dirified, request.action, request.params)
+      response:write("redirect")
+      return response
+   end
   
    if stub and self.config.PROTOTYPE_PATTERNS then
       -- If an empty stub was returned, check the PROTOTYPE_PATTERNS table to
@@ -736,52 +781,39 @@ function Sputnik:run(request, response)
 
    if not action_function then
       content, content_type = sputnik.actions.wiki.actions.action_not_found(node, request, self)
-   else
+   elseif node:check_permissions(request.user, action) then
       -- Check permissions on the node, for the given action
-      if node:check_permissions(request.user, action) then
-         -- Determine if there are any hooks to be called for this action, on this node
-         -- by checking the node.action_hooks table
-         if node.action_hooks and node.action_hooks[action] then
-            local hooks = node.action_hooks[action].before
-            if hooks then
-               for idx, hook in ipairs(hooks) do
-                  local mod_name, dot_action = sputnik.util.split(hook, "%.")
-                  local mod = require("sputnik.actions." .. mod_name)
-                  if mod and mod.actions and mod.actions[dot_action] then
-                     pcall(mod.actions[dot_action], node, request, sputnik)
-                  end
-               end
-            end
-         end
+      content, content_type = action_function(node, request, self)
+   else
+      -- The user did not have permission, so give a message stating this
+      node:post_error("Sorry, that action is not allowed")
+      node.inner_html = ""
+      content, content_type = node.wrappers.default(node, request, self)
+   end
 
-         content, content_type = action_function(node, request, self)
+   if request.redirect then
+      self.logger:debug("Sending redirect")
+      -- Set the authentication cookie so it's not lost
+      local cookie_value = (request.user or "").."|"..(request.auth_token or "")
+      response:set_cookie(self.cookie_name, {value=cookie_value, path="/"})
 
-         -- Handle any action hooks at this point, with no digging for post actions
-         -- If you want to hook a post action, you need to iterate the parameters 
-         -- to determine which action is actually being called
-         if node.action_hooks and node.action_hooks[action] then
-            local hooks = node.action_hooks[action].after
-            if hooks then
-               for idx, hook in ipairs(hooks) do
-                  local mod_name, dot_action = sputnik.util.split(hook, "%.")
-                  local mod = require("sputnik.actions." .. mod_name)
-                  if mod and mod.actions and mod.actions[dot_action] then
-                     pcall(mod.actions[dot_action], node, request, sputnik)
-                  end
-               end
-            end
-         end
-      else
-         -- The user did not have permission, so give a message stating this
-         node:post_error("Sorry, that action is not allowed")
-         node.inner_html = ""
-         content, content_type = node.wrappers.default(node, request, self)
-      end
+      response.headers["Content-Type"] = "text/html"
+      response.headers["Location"] = request.redirect
+      response:write("redirect")
+      return response
    end
 
    assert(content)
    response.headers["Content-Type"] = content_type or "text/html"
 
+   if node.http_cache_control and node.http_cache_control~="" then
+      node.headers["Cache-Control"] = node.http_cache_control
+   end
+   if node.http_expires and node.http_expires~="" then
+      node.headers["Expires"] = os.date("!%a, %d %b %Y %H:%M:%S GMT",
+                                        os.time()+3600*1000*tonumber(node.http_expires))
+   end
+
    -- If we have any custom HTML headers, add them to the response
    for header,value in pairs(node.headers) do
       response.headers[header] = value
@@ -789,78 +821,23 @@ function Sputnik:run(request, response)
 
    local cookie_value = (request.user or "").."|"..(request.auth_token or "")
    response:set_cookie(self.cookie_name, {value=cookie_value, path="/"})
-   response:write(content)
+
+   -- gzip compression code borrowed from Ignacio Burgeno
+   if self.config.USE_COMPRESSION and zlib_loaded
+      and string.find(request.wsapi_env["HTTP_ACCEPT_ENCODING"], "gzip") then
+      --zlib.compress(string buffer [, int level] [, int method] [, int windowBits] [, int memLevel] [, int strategy])
+      --that magic came from http://lua-users.org/lists/lua-l/2005-03/msg00221.html
+      local gz_content = zlib.compress(content, 9, nil, 15 + 16)
+      response.headers["Content-Encoding"] = "gzip"
+      response.headers["Content-Length"] = #gz_content
+      response.headers["Vary"] = "accept-encoding"
+      response:write(gz_content)
+   else
+      response:write(content)
+   end
+
+   if self.saci.reset_cache then self.saci:reset_cache() end
    return response
 end
 
------------------------------------------------------------------------------
--- Handles a request, throwing errors if something goes wrong.
------------------------------------------------------------------------------
-function Sputnik:unprotected_run(request, response)
-   return self:run(request, response)
-end
-
-
------------------------------------------------------------------------------
--- Handles a request safely.
------------------------------------------------------------------------------
-function Sputnik:protected_run(request, response)
-   local function mypcall(fn, ...)
-      local params = {...} -- to keep the inner function from being confused
-      return xpcall(function()  return fn(unpack(params)) end,
-                    function(e) return {e, require("debug").traceback()} end )
-   end  
-
-   local success, err = mypcall(self.unprotected_run, self, request, response)
-   if success then
-      return success
-   else
-      local message = "Sputnik ran but failed due to an unexpected error." -- ::LOCALIZE::
-
-      -- catch some common errors
-      local dummy, path = string.match(err[1], "Versium storage error: (.*) Can't open file: (.*) in mode w")
-      local dir = self.config.VERSIUM_PARAMS.dir
-      if path and path:sub(1, dir:len()) == dir then
-         message = "Versium's data directory ("..dir
-                   ..") is not writable.<br/> Please fix directory permissions." -- ::LOCALIZE::
-      end
-
-      return success, string.format([[
-       <br/>
-       <span style="color:red; font-size: 19pt;">%s</span></br><br/><br/>
-       Error details: <pre><b><code>%s</code></b></pre><br/>
-       <pre><code>%s</code></pre>
-      ]], message, err[1], err[2])
-   end
-end
-
------------------------------------------------------------------------------
--- Handles a request coming from WSAPI
------------------------------------------------------------------------------
-
-function Sputnik:wsapi_run(wsapi_env)
-
-   _G.format = string.format -- to work around a bug in wsapi.response
-
-   require("wsapi.request")
-   local request = wsapi.request.new(wsapi_env)
-   request.wsapi_env = wsapi_env
-
-   require("wsapi.response")
-   local response = wsapi.response.new()
-
-   local success, error_message = self:protected_run(request, response)
-   if not success then
-      response = wsapi.response.new()
-      response:write(error_message)
-   end
-   -- Change the HTTP status code to 302 is a location header is set
-   if response.headers["Location"] then
-      if response.status < 300 then
-         response.status = 302
-      end
-   end
-   return response:finish()
-end
-
 -- vim:ts=3 ss=3 sw=3 expandtab
