@@ -9,7 +9,9 @@ show = "binaryfile.show"
 save = "binaryfile.save"
 download = "binaryfile.download"
 ]=],
-permissions = [=[
+   http_cache_control = "max-age=3600",
+   http_expires = "2",
+   permissions = [=[
 allow(all_users, "jpg")
 allow(all_users, "gif")
 allow(all_users, "png")
@@ -42,8 +44,8 @@ content = nil
 file_upload = {1.30, "file"}
 file_description = {1.31, "text_field"}
 file_copyright = {1.32, "text_field"}
-file_name = {1.33, "text_field", advanced = true}
-file_type = {1.34, "text_field", advanced = true}
-file_size = {1.35, "text_field", advanced = true}
+file_name = {1.33, "text_field"}
+file_type = {1.34, "text_field"}
+file_size = {1.35, "text_field"}
 ]=],
 }
