@@ -20,10 +20,22 @@ FALLBACKS = {
 -----------------------------------------------------------------------------
 
 -- The login link
-LOGIN_OR_REGISTER = {
-   en_US = "<a $login_link>Login</a> or <a $register_link>register</a>",
-   ru    = "<a $login_link>Представиться</a> или <a $register_link>зарегистрироваться</a>",     
-   pt_BR = "<a $login_link>Login</a> | <a $register_link>Criar conta</a>",
+LOGIN = {
+   en_US = "Login",
+   ru    = "Представиться",     
+   pt_BR = "Login",
+}
+
+OR = {
+   en_US = "or",
+   ru    = "или",     
+   pt_BR = "ou",
+}
+
+REGISTER = {
+   en_US = "register",
+   ru    = "зарегистрироваться",     
+   pt_BR = "Criar conta",
 }
 
 -- The login button
@@ -62,6 +74,13 @@ HISTORY = {
    pt_BR = "Histórico",
 }
 
+-- The link to the node configurations
+CONFIGURE = {
+   en_US = "Configure",
+   ru    = "Настройки",     
+   pt_BR = "Configuração",
+}
+
 -- The "powered by sputnik" label in the footer.
 POWERED_BY_SPUTNIK = {
    en_US = "Powered by <a $sputnik_link>Sputnik</a>",
@@ -89,6 +108,24 @@ TOOLTIP_FOR_SEARCH = {
    pt_BR = "Busca nessa wiki",
 }
 
+TOOLTIP_FOR_SEARCH_BOX = {
+   en_US = "Search query",
+   ru    = "Термины поиска",
+}
+
+CURRENT_SECTION = {
+   en_US = "Current section"
+}
+
+CURRENT_SUBSECTION = {
+   en_US = "Current subsection"
+}
+
+CURRENT_PAGE = {
+   en_US = "Current page"
+}
+
+
 -----------------------------------------------------------------------------
 -----  MISC ACTIONS  --------------------------------------------------------
 -----------------------------------------------------------------------------
@@ -153,6 +190,10 @@ DIFF_SELECTED_VERSIONS = {
    en_US = "Diff selected versions",
 }
 
+SHOW_CHANGES_SINCE_PREVIOUS = {
+   en_US = "Show changes since previous version",
+}
+
 -----------------------------------------------------------------------------
 -----  EDITING: BASIC -------------------------------------------------------
 -----------------------------------------------------------------------------
@@ -178,18 +219,75 @@ EDIT_FORM_TITLE = {
    pt_BR = "Título da Página",
 }
 
+-- A label for the section of the form with advanced parameters
+EDIT_FORM_ADVANCED_SECTION = {
+   en_US = "Advanced Fields",
+   ru    = "Дополнительные поля",
+   pt_BR = "Campos avançados",
+}
+
+-- A label for the section of the form with even more advanced parameters
+EDIT_FORM_GURU_SECTION = {
+   en_US = "Guru Fields",
+   ru    = "Поля для гуру",
+   pt_BR = "Campos ainda mais avançados",
+}
+
+-- A label for the section of the form with html templates
+EDIT_FORM_HTML_SECTION = {
+   en_US = "HTML Fields",
+   ru    = "Поля HTML",
+   pt_BR = "Campos HTML",
+}
+
+EDIT_FORM_HTML_MAIN = {
+   en_US = "Main",
+}
+EDIT_FORM_HTML_HEAD = {
+   en_US = "Head",
+}
+EDIT_FORM_HTML_BODY = {
+   en_US = "Body",
+}
+EDIT_FORM_HTML_HEADER = {
+   en_US = "Header",
+}
+EDIT_FORM_HTML_MENU = {
+   en_US = "Menu",
+}
+EDIT_FORM_HTML_LOGO = {
+   en_US = "Logo",
+}
+EDIT_FORM_HTML_SEARCH = {
+   en_US = "Search",
+}
+EDIT_FORM_HTML_PAGE = {
+   en_US = "Page",
+}
+EDIT_FORM_HTML_SIDEBAR = {
+   en_US = "Sidebar",
+}
+EDIT_FORM_HTML_FOOTER = {
+   en_US = "Footer",
+}
+
 -- A label for the section of the form where the user will edit the page content
-EDIT_FORM_CONTENT_HDR = {
+EDIT_FORM_CONTENT_SECTION = {
    en_US = "Page Content",
    ru    = "Содержание страницы",
    pt_BR = "Conteúdo da página",
 }
 
-EDIT_FORM_CONTENT = EDIT_FORM_CONTENT_HDR
+EDIT_FORM_CONTENT = EDIT_FORM_CONTENT_SECTION
+
+-- A label for the section of the form where the user can edit the breadcrumb text
+EDIT_FORM_BREADCRUMB = {
+	en_US = "Breadcrumb Text",
+}
 
 -- A lable for the section of the form that deals with user id, minor parameter and edit summary
-EDIT_FORM_EDIT_INFO_HDR = {
-   en_US = "About this Edit",
+EDIT_FORM_EDIT_INFO_SECTION = {
+   en_US = "About This Edit",
    ru    = "О ваших изменениях",
    pt_BR = "Sobre esta edição",
 }
@@ -243,6 +341,9 @@ EDIT_FORM_FILE_COPYRIGHT = {
 	en_US = "Copyright",
 }
 
+EDIT_FORM_SUBJECT = {
+   en_US = "Subject",
+}
 -----------------------------------------------------------------------------
 -----  EDITING: ADMIN -------------------------------------------------------
 -----------------------------------------------------------------------------
@@ -258,7 +359,7 @@ SHOW_ADVANCED_OPTIONS = {
 HIDE_ADVANCED_OPTIONS = {
    en_US = "Hide Advanced Options",
    ru    = "спрятать дополнительный опции",
-   pt_BR = "Tirar Opções Avançadas",
+   pt_BR = "Ocultar Opções Avançadas",
 }
 
 -- A label for the "category" parameter
@@ -275,6 +376,11 @@ EDIT_FORM_PROTOTYPE = {
    pt_BR = "Prototipo",
 }
 
+-- A label for the "save_hook" parameter
+EDIT_FORM_SAVE_HOOK = {
+   en_US = "Save Hook"
+}
+
 -- A label for the "templates" parameter
 EDIT_FORM_TEMPLATES = {
    en_US = "Templates",
@@ -309,6 +415,12 @@ EDIT_FORM_CONFIG = {
    ru    = "Прочие настройки",
 }
 
+EDIT_FORM_MARKUP_MODULE = {
+   en_US = "Markup Module",
+   ru    = "Модуль разметки",
+}
+
+
 EDIT_FORM_FIELDS = {
    en_US = "Fields",
    ru    = "Поля",
@@ -325,6 +437,77 @@ EDIT_FORM_ADMIN_EDIT_UI = {
    ru    = "Редактирование для админа",
 }
 
+EDIT_FORM_CHILD_PROTO = {
+   en_US = "Child Prototype"
+}
+
+EDIT_FORM_COLLECTION_SECTION = {
+   en_US = "Collection Fields"
+}
+
+EDIT_FORM_CHILD_UID_FORMAT = {
+    en_US = "Child UID Format"
+}
+
+EDIT_FORM_DISC_SECTION = {
+   en_US = "Discussion Fields"
+}
+
+EDIT_FORM_AUTHOR = {
+   en_US = "Author"
+}
+
+EDIT_FORM_CREATION_TIME = {
+   en_US = "Creation Time",
+}
+
+EDIT_FORM_ACTIVITY_TIME = {
+   en_US = "Activity Time",
+}
+
+EDIT_FORM_ACTIVITY_NODE = {
+   en_US = "Activity Node",
+}
+
+EDIT_FORM_COMMENT_AUTHOR = {
+   en_US = "Comment Author",
+}
+
+EDIT_FORM_COMMENT_PARENT = {
+    en_US = "Comment Parent",
+}
+
+EDIT_FORM_COMMENT_SECTION = {
+    en_US = "Comment Fields"
+}
+
+EDIT_FORM_CONTENT_TEMPLATE = {
+   en_US = "Content Template"
+}
+
+EDIT_FORM_HTML_CONTENT = {
+   en_US = "Content Template"
+}
+
+EDIT_FORM_XML_TEMPLATE = {
+   en_US = "XML Template"
+}
+
+EDIT_FORM_HTTP_SECTION = {
+   en_US = "HTTP Fields"
+}
+
+EDIT_FORM_HTTP_CACHE_CONTROL = {
+   en_US = "Cache-Control"
+}
+
+EDIT_FORM_HTTP_EXPIRES = {
+   en_US = "Expires"
+}
+
+EDIT_FORM_XSSFILTER_ALLOWED_TAGS = {
+   en_US = "Tags Allowed for XSSFilter"
+}
 
 -----------------------------------------------------------------------------
 -----  LOGIN  ---------------------------------------------------------------
@@ -350,6 +533,7 @@ EDIT_FORM_PASSWORD = {
    pt_BR = "Senha",
 }
 
+
 -- A note to the user that they must be logged in to edit
 YOU_MUST_BE_LOGGED_IN = {
    en_US = "You must be logged in to edit a page!  Try again!",
@@ -415,7 +599,7 @@ PREVIEW = {
 SAVE = {
    en_US = "save",
    ru    = "сохранить",
-   pt_BR = "Salvar",
+   pt_BR = "Guardar",
 }
 
 -- The label for the "cancel" button in the edit forms. 
@@ -481,25 +665,25 @@ RSS_FOR_EDITS_TO_THIS_NODE = {
    ru    = "RSS-лента изменение этой станицы",
 }
 
-LARGE_RSS_ICON = { 
-   en_US = "Large RSS Icon",
-   ru    = "Большой значок RSS"
+RSS = RSS_FOR_EDITS_TO_THIS_NODE
+
+BUTTON = {
+   en_US = "Button",
+   ru    = "Кнопка",
 }
 
-SMALL_RSS_ICON = { 
-   en_US = "Small RSS Icon",
-   ru    = "Maленький значок RSS"
+EDIT_FORM_HTML_META_KEYWORDS = {
+   en_US = "HTML/Meta/Keywords",
 }
 
-EDIT_ICON = {
-   en_US = "Edit Icon",
+EDIT_FORM_HTML_META_DESCRIPTION = {
+   en_US = "HTML/Meta/Description",
 }
 
-HISTORY_ICON = {
-   en_US = "History Icon",
+EDIT_FORM_REDIRECT_DESTINATION = {
+	en_US = "Redirect",
 }
 
-
 -----------------------------------------------------------------------------
 -----  REGISTRATION  --------------------------------------------------------
 -----------------------------------------------------------------------------
@@ -551,6 +735,11 @@ EDIT_FORM_NEW_EMAIL = {
    en_US = "Email"
 }
 
+-- Form label for terms of service
+EDIT_FORM_AGREE_TOS = {
+   en_US = "Terms of Service"
+}
+
 -- Form label for confirming new password
 EDIT_FORM_NEW_PASSWORD_CONFIRM = {
    en_US = "Confirm Password"
@@ -564,6 +753,10 @@ NEW_EMAIL_NOT_VALID = {
    en_US = "The email you entered is not valid."
 }
 
+MUST_CONFIRM_TOS = {
+   en_US = "You must agree to the terms of service in order to create a new account.",
+}
+
 USERNAME_TAKEN = {
    en_US = "This user name is already taken."
 }
@@ -596,5 +789,49 @@ ANONYMOUS_USERS_MUST_ENTER_CAPTCHA = {
    en_US = "Anonymous users must enter <a href='http://en.wikipedia.org/wiki/Captcha'>captcha</a> below."
 }
 
+NO_SUCH_NODE = {
+   en_US = "This node by this name does not exist."
+}
+
+PLEASE_PICK_A_TYPE_TO_CREATE_A_NEW_NODE = {
+   en_US = [[The node by this name does not exist, but you can create it.
+             Please pick one of the node types below.]]
+}
+
+THIS_PAGE_DEFINED_THE_FOLLOWING_ACTIONS = {
+   en_US = "This node supports the following commands:"
+}
+
+-----------------------------------------------------------------------------
+-----  RSS OUTPUT  --------------------------------------------------------
+-----------------------------------------------------------------------------
+
+NO_EDIT_SUMMARY = {
+   en_US = "No edit summary provided",
+}
+
+ANONYMOUS_USER = {
+   en_US = "Anonymous user",
+}
+
+-----------------------------------------------------------------------------
+-----  COMMENTS AND FORUMS  -------------------------------------------------
+-----------------------------------------------------------------------------
+
+REPLY = {
+   en_US = "Reply",
+   pt_BR = "Responder",
+   ru = "Ответить"
+}
+
+QUOTE = {
+   en_US = "Quote",
+   ru = "Цитировать"
+}
+
+ADD_NEW_DISCUSSION_TOPIC = {
+   en_US = "Add new discussion topic",
+   ru = "Добавить тему",
+}
 
 ]=============]
